@@ -130,6 +130,40 @@ class FortiManager:
         get_adoms = session.post(url=self.base_url, json=payload, verify=self.verify)
         return get_adoms.json()["result"]
 
+    def __lock_unlock_adom(self, method, name=False):
+        """
+        Lock or Unlock current Adom in FortiManager 
+        Adom has to be in workspace mode
+        :param method: lock or unlock adom
+        :param name: Can lock specific adom using name as a filter
+        :return: Response of status code (0=success) with data in JSON Format
+        """
+        url = "dvmdb/adom"
+
+        if name:
+            url = f"dvmdb/adom/{name}/workspace/{method}"
+        else:
+            url = f"dvmdb/adom/{self.adom}/workspace/{method}"
+
+        payload = \
+            {
+                "method": "exec",
+                "params":
+                    [
+                        {
+                            "url": url
+                        }
+                    ],
+            }
+
+        return self.custom_api(payload)
+
+    def lock_adom(self, name=False):
+        return self.__lock_unlock_adom("lock", name)
+
+    def unlock_adom(self, name=False):
+        return self.__lock_unlock_adom("unlock", name)
+
     def get_devices(self):
         """
         :return: returns list of devices added in FortiManager
@@ -313,7 +347,7 @@ class FortiManager:
         """
         session = self.login()
         payload = {"method": "update",
-                   "params": [{"url": f"/dvmdb/adom/root/device/{device}",
+                   "params": [{"url": f"/dvmdb/adom/{self.adom}/device/{device}",
                                "data": {"name": f"{device}", "meta fields": {f"{meta_name}": f"{meta_value}"}}}]}
         payload.update({"session": self.sessionid})
         assign_meta = session.post(
@@ -331,7 +365,7 @@ class FortiManager:
         """
         session = self.login()
         payload = {"method": "update",
-                   "params": [{"url": f"/dvmdb/adom/root/device/{device}/vdom/{vdom}",
+                   "params": [{"url": f"/dvmdb/adom/{self.adom}/device/{device}/vdom/{vdom}",
                                "data": {"name": f"{device}", "meta fields": {f"{meta_name}": f"{meta_value}"}}}]}
         payload.update({"session": self.sessionid})
         assign_meta_vdom = session.post(
@@ -412,7 +446,7 @@ class FortiManager:
         add_address_object = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return add_address_object.json()["result"]
-    
+
     def add_firewall_address_v6_object(self, name, subnet6: str, object_type=0):
         """
         Create an address object using provided info
@@ -424,7 +458,7 @@ class FortiManager:
         session = self.login()
         payload = {
             "method": "add",
-            "params": [{"data": {                                
+            "params": [{"data": {
                 "name": name,
                 "ip6": subnet6,
                 "type": object_type},
@@ -601,7 +635,7 @@ class FortiManager:
         get_address_group = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return get_address_group.json()["result"]
-    
+
     def get_address_v6_groups(self, name=False):
         """
         Get the address groups created in your FortiManager
@@ -625,7 +659,7 @@ class FortiManager:
         get_address_group = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return get_address_group.json()["result"]
-    
+
     def add_address_group(self, name, members=list):
         """
         Create your own group with just 2 parameters
@@ -651,7 +685,7 @@ class FortiManager:
         add_address_group = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return add_address_group.json()["result"]
-    
+
     def add_address_v6_group(self, name, members=list):
         """
         Create your own group with just 2 parameters
@@ -677,7 +711,7 @@ class FortiManager:
         add_address_group = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return add_address_group.json()["result"]
-    
+
     def update_address_group(self, name, object_name, do="add"):
         """
         Update Members of the Address group
@@ -859,6 +893,79 @@ class FortiManager:
         get_firewall_header_policies = session.post(url=self.base_url, json=payload, verify=self.verify)
         return get_firewall_header_policies.json()["result"]
 
+    # Footer
+    def get_global_footer_policies(self, policy_package_name="default", policyid=False):
+        """
+        Get global footer policies
+        """
+        url = f"pm/config/global/pkg/{policy_package_name}/global/footer/policy"
+        if policyid:
+            url = url + str(policyid)
+        session = self.login()
+        payload = {
+            "method": "get",
+            "params": [
+                {
+                    "url": url
+                }
+            ],
+            "session": self.sessionid
+        }
+        get_global_footer_policies = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return get_global_footer_policies.json()["result"]
+
+    def get_firewall_footer_policies(self, policy_package_name="default", policyid=False):
+        """
+        Get adom footer policies
+        """
+        url = f"pm/config/adom/{self.adom}/obj/global/footer/policy"
+        if policyid:
+            url = url + str(policyid)
+        session = self.login()
+        payload = {
+            "method": "get",
+            "params": [
+                {
+                    "url": url
+                }
+            ],
+            "session": self.sessionid
+        }
+        get_firewall_footer_policies = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return get_firewall_footer_policies.json()["result"]
+
+    # Policy Lookup
+    def policy_lookup(self, device, source_interface, source_ip, destination_ip, protocol, port, vdom="root"):
+        session = self.login()
+        payload = {"method": "exec",
+                   "params": [{"url": "sys/proxy/json",
+                               "data": {
+                                   "target": [f"adom/{self.adom}/device/{device}"],
+                                   "action": "get",
+                                   "resource": f"/api/v2/monitor/firewall/policy-lookup/select?vdom={vdom}"
+                                               f"&srcintf={source_interface}"
+                                               f"&protocol={protocol}"
+                                               f"&sourceip={source_ip}"
+                                               f"&sourceport="
+                                               f"&dest={destination_ip}"
+                                               f"&destport={port}"}}]}
+        payload.update(session=self.sessionid)
+        req = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return req.json()["result"]
+
+    def get_policies_assigned_to_device(self, device, vdom):
+        session = self.session
+        payload = {
+            "method": "exec",
+            "params": [
+                {"url": "sys/proxy/json",
+                 "data": {"target": [f"adom/root/device/{device}"],
+                          "action": "get",
+                          "resource": f"/api/v2/cmdb/firewall/policy/?vdom={vdom}"}}]}
+        payload.update(session=self.sessionid)
+        req = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return req.json()["result"]
+
     # Firewall Interfaces
     def get_device_interfaces(self, device):
         """
@@ -877,6 +984,21 @@ class FortiManager:
         get_interfaces = session.post(url=self.base_url, json=payload, verify=self.verify)
         return get_interfaces.json()["result"]
 
+    def get_interfaces(self, device):
+        session = self.login()
+        payload = {"method": "get", "params": [{"url": f"pm/config/device/{device}/global/system/interface"}]}
+        payload.update(session=self.sessionid)
+        req = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return req.json()["result"]
+
+    def get_interface(self, device, interface):
+        session = self.login()
+        payload = {"method": "get",
+                   "params": [{"url": f"pm/config/device/{device}/global/system/interface/{interface}"}]}
+        payload.update(session=self.sessionid)
+        req = session.post(url=self.base_url, json=payload, verify=self.verify)
+        return req.json()["result"]
+
     # Services
     def get_services(self):
         """
@@ -929,7 +1051,7 @@ class FortiManager:
         get_firewall_policies = session.post(
             url=self.base_url, json=payload, verify=self.verify)
         return get_firewall_policies.json()["result"]
-	
+
     def get_dhcp(self, device):
         """
         Get dhcp details from the devices.
@@ -945,7 +1067,7 @@ class FortiManager:
                               "resource": "/api/v2/monitor/system/dhcp/select?&vdom=root&ipv6=true&scope=global"}}]}
         payload.update(session=self.sessionid)
         get_interfaces = session.post(url=self.base_url, json=payload, verify=self.verify)
-        return get_interfaces.json()["result"] 
+        return get_interfaces.json()["result"]
 
     def add_firewall_policy(self, policy_package_name: str, name: str, source_interface: str,
                             source_address: str, destination_interface: str, destination_address: str,
@@ -995,8 +1117,9 @@ class FortiManager:
         return add_policy.json()
 
     def add_firewall_policy_with_v6(self, policy_package_name: str, name: str, source_interface: str,
-                            source_address: Any, source_address6: Any, destination_interface: str, destination_address: Any, destination_address6: Any,
-                            service: str, nat='disable', schedule="always", action=1, logtraffic=2):
+                                    source_address: Any, source_address6: Any, destination_interface: str,
+                                    destination_address: Any, destination_address6: Any,
+                                    service: str, nat='disable', schedule="always", action=1, logtraffic=2):
         """
         Create your own policy in FortiManager using the instance parameters.
         :param policy_package_name: Enter the name of the policy package                eg. "default"
@@ -1024,14 +1147,14 @@ class FortiManager:
                 {
                     "data": {
                         "dstaddr": destination_address,
-                        "dstaddr6" : destination_address6,
+                        "dstaddr6": destination_address6,
                         "dstintf": destination_interface,
                         "logtraffic": logtraffic,
                         "name": name,
                         "schedule": schedule,
                         "service": service,
                         "srcaddr": source_address,
-                        "srcaddr6" : source_address6,
+                        "srcaddr6": source_address6,
                         "srcintf": source_interface,
                         "action": action,
                         "nat": nat
@@ -1384,7 +1507,7 @@ class FortiManager:
                     "data": {"adom": self.adom,
                              "scope": devices,
                              "script": script_name},
-                    "url": "/dvmdb/adom/root/script/execute"}],
+                    "url": f"/dvmdb/adom/{self.adom}/script/execute"}],
                 "session": self.sessionid
             }
         run_script = session.post(
@@ -1407,7 +1530,7 @@ class FortiManager:
                     "data": {"adom": self.adom,
                              "scope": {"name": device_name, "vdom": vdom},
                              "script": script_name},
-                    "url": "/dvmdb/adom/root/script/execute"}],
+                    "url": f"/dvmdb/adom/{self.adom}/script/execute"}],
                 "session": self.sessionid
             }
 
